@@ -14,7 +14,7 @@ from functools import reduce
 from http.client import HTTPException
 from os.path import splitext
 from threading import Thread
-from typing import Dict, Collection, Callable, Iterable
+from typing import Dict, Collection, Callable, Iterable, Optional
 from urllib.request import urlopen
 
 import re
@@ -124,6 +124,10 @@ class IRFile(RemoteFile):
                 lines.append(encode(tag) + b"=" + encode(value))
         return b"\n".join(lines)
 
+    @property
+    def lyric_filename(self) -> Optional[str]:
+        return None
+
     def can_change(self, k=None):
         if self.streamsong:
             if k is None:
@@ -828,7 +832,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
     def __add_stations(self, irfs: Collection[IRFile], uri: str) -> None:
         print_d(f"Got {len(irfs)} station(s) from {uri}")
-        assert self.__fav_stations
+        assert self.__fav_stations is not None
         if not irfs:
             msg = ErrorMessage(
                 self, _("No stations found"),
@@ -970,9 +974,8 @@ class InternetRadio(Browser, util.InstanceTracker):
         self.view.set_cursor(path)
         self.view.scroll_to_cell(path, use_align=True, row_align=0.5)
 
-    def status_text(self, count, time=None):
-        return numeric_phrase("%(count)d station", "%(count)d stations",
-                              count, 'count')
+    def status_text(self, count: int, time: Optional[str] = None) -> str:
+        return numeric_phrase("%(count)d station", "%(count)d stations", count, 'count')
 
 
 from quodlibet import app
