@@ -1,20 +1,18 @@
-# -*- coding: utf-8 -*-
-# Copyright 2004-2013 Joe Wreschnig, Michael Urman, Niklas Janlert,
-#                     Steven Robertson, Nick Boultbee
+# Copyright 2004-2020 Joe Wreschnig, Michael Urman, Niklas Janlert,
+#                     Steven Robertson, Nick Boultbee, h88e22dgpeps56sg
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 import mutagen.id3
 
-from quodlibet import config, const, print_d
+from quodlibet import config, const, print_w
 from quodlibet import util
-from quodlibet.compat import iteritems, text_type, listvalues, PY2
 from quodlibet.util.iso639 import ISO_639_2
 from quodlibet.util.path import get_temp_cover_file
 from quodlibet.util.string import isascii
-
 from ._audio import AudioFile, translate_errors, AudioFileError
 from ._image import EmbeddedImage, APICType
 
@@ -24,14 +22,15 @@ def encoding_for(s):
     return 3 if isascii(s) else 1
 
 
-RG_KEYS = [
+RG_KEYS = {
     "replaygain_track_peak", "replaygain_track_gain",
     "replaygain_album_peak", "replaygain_album_gain",
-]
+}
 
 
 # ID3 is absolutely the worst thing ever.
 class ID3File(AudioFile):
+    supports_rating_and_play_count_in_file = True
 
     # http://www.unixgods.org/~tilo/ID3/docs/ID3_comparison.html
     # http://www.id3.org/id3v2.4.0-frames.txt
@@ -67,10 +66,11 @@ class ID3File(AudioFile):
            "TSOC": "composersort",
            "TMED": "media",
            "TCMP": "compilation",
+           "TKEY": "initialkey",
            # TLAN requires an ISO 639-2 language code, check manually
            #"TLAN": "language"
     }
-    SDI = dict([(v, k) for k, v in iteritems(IDS)])
+    SDI = dict((v, k) for k, v in IDS.items())
 
     # At various times, information for this came from
     # http://musicbrainz.org/docs/specs/metadata_tags.html
@@ -93,7 +93,7 @@ class ID3File(AudioFile):
         u"ALBUMARTISTSORT": "albumartistsort",
         u"BARCODE": "barcode",
         }
-    PAM_XXXT = dict([(v, k) for k, v in iteritems(TXXX_MAP)])
+    PAM_XXXT = dict((v, k) for k, v in TXXX_MAP.items())
 
     Kind = None
 
@@ -113,19 +113,9 @@ class ID3File(AudioFile):
             elif frame.FrameID == "TCON":
                 self["genre"] = "\n".join(frame.genres)
                 continue
-            elif frame.FrameID == "TLEN":
-                try:
-                    length = +frame / 1000.0
-                except ValueError:
-                    continue
-                # ignore TLEN <= 0 [issue 222]
-                if length > 0:
-                    self["~#length"] = length
-                continue
             elif (frame.FrameID == "UFID" and
                   frame.owner == "http://musicbrainz.org"):
-                self["musicbrainz_trackid"] = frame.data.decode("utf-8",
-                                                                "replace")
+                self["musicbrainz_trackid"] = frame.data.decode("utf-8", "replace")
                 continue
             elif frame.FrameID == "POPM":
                 rating = frame.rating / 255.0
@@ -163,8 +153,7 @@ class ID3File(AudioFile):
             elif frame.FrameID == "TLAN":
                 self["language"] = "\n".join(frame.text)
                 continue
-            elif (frame.FrameID == "USLT" and frame.desc == "" and
-                  frame.lang == "\x00\x00\x00"):
+            elif frame.FrameID == "USLT":
                 name = "lyrics"
             else:
                 name = self.IDS.get(frame.FrameID, "").lower()
@@ -176,12 +165,14 @@ class ID3File(AudioFile):
 
             id3id = frame.FrameID
             if id3id.startswith("T"):
-                text = "\n".join(map(text_type, frame.text))
+                text = "\n".join(map(str, frame.text))
             elif id3id == "COMM":
                 text = "\n".join(frame.text)
             elif id3id == "USLT":
                 # lyrics are single string, not list
                 text = frame.text
+                self["~lyricsdescription"] = frame.desc
+                self["~lyricslanguage"] = frame.lang
             elif id3id.startswith("W"):
                 text = frame.url
                 frame.encoding = 0
@@ -208,13 +199,13 @@ class ID3File(AudioFile):
         # to avoid reverting or duplicating tags in existing libraries.
         if audio.tags and "date" not in self:
             for frame in tag.getall('TXXX:DATE'):
-                self["date"] = "\n".join(map(text_type, frame.text))
+                self["date"] = "\n".join(map(str, frame.text))
 
         # Read TXXX replaygain and replace previously read values from RVA2
         for frame in tag.getall("TXXX"):
             k = frame.desc.lower()
             if k in RG_KEYS:
-                self[str(k)] = u"\n".join(map(text_type, frame.text))
+                self[str(k)] = u"\n".join(map(str, frame.text))
 
         self.sanitize(filename)
 
@@ -233,10 +224,7 @@ class ID3File(AudioFile):
                 and k.encode("ascii", "replace").decode("ascii") == k):
             return
 
-        if PY2:
-            return k.encode("ascii")
-        else:
-            return k
+        return k
 
     def __process_rg(self, frame):
         if frame.channel == 1:
@@ -260,7 +248,7 @@ class ID3File(AudioFile):
         return codecs
 
     def __distrust_latin1(self, text, encoding):
-        assert isinstance(text, text_type)
+        assert isinstance(text, str)
         if encoding == 0:
             try:
                 text = text.encode('iso-8859-1')
@@ -279,6 +267,13 @@ class ID3File(AudioFile):
                 return None
         return text
 
+    def has_rating_and_playcount_in_file(self, email):
+        with translate_errors():
+            audio = self.Kind(self['~filename'])
+        if audio.tags is None:
+            return False
+        return ("POPM:" + email) in audio.tags
+
     def write(self):
         with translate_errors():
             audio = self.Kind(self['~filename'])
@@ -318,8 +313,9 @@ class ID3File(AudioFile):
             else:
                 tag.add(Kind(encoding=enc, text=text))
 
-        dontwrite = ["genre", "comment", "musicbrainz_trackid", "lyrics"] \
-            + RG_KEYS + listvalues(self.TXXX_MAP)
+        dont_write = (RG_KEYS
+                      | set(self.TXXX_MAP.values())
+                      | {"genre", "comment", "musicbrainz_trackid", "lyrics"})
 
         if "musicbrainz_trackid" in self.realkeys():
             f = mutagen.id3.UFID(
@@ -334,18 +330,19 @@ class ID3File(AudioFile):
             if all([lang in ISO_639_2 for lang in langs]):
                 # Save value(s) to TLAN tag. Guaranteed to be ASCII here
                 tag.add(mutagen.id3.TLAN(encoding=3, text=langs))
-                dontwrite.append("language")
+                dont_write.add("language")
             else:
-                print_d("Not using invalid language code '%s' in TLAN" %
-                        self["language"])
+                print_w(
+                    f"Not using invalid language {self['language']!r} in TLAN")
 
         # Filter out known keys, and ones set not to write [generically].
-        keys_to_write = filter(lambda k: not (k in self.SDI or k in dontwrite),
-                               self.realkeys())
+        dont_write |= self.SDI.keys()
+        keys_to_write = (k for k in self.realkeys()
+                         if k not in dont_write)
         for key in keys_to_write:
             enc = encoding_for(self[key])
             if key.startswith("performer:"):
-                mcl.people.append([key.split(":", 1)[1], self[key]])
+                mcl.people.append((key.split(":", 1)[1], self[key]))
                 continue
 
             f = mutagen.id3.TXXX(
@@ -373,12 +370,17 @@ class ID3File(AudioFile):
             tag.add(mutagen.id3.COMM(encoding=enc, text=t, desc=u"",
                                      lang="\x00\x00\x00"))
 
-        tag.delall("USLT::\x00\x00\x00")
+        tag.delall("USLT")
         if "lyrics" in self:
             enc = encoding_for(self["lyrics"])
+            if not ("~lyricslanguage" in self and
+                    # language has to be a 3 byte ISO 639-2 code
+                    self["~lyricslanguage"] in ISO_639_2):
+                self["~lyricslanguage"] = "und" # undefined
             # lyrics are single string, not array
             tag.add(mutagen.id3.USLT(encoding=enc, text=self["lyrics"],
-                                     desc=u"", lang="\x00\x00\x00"))
+                                     desc=self.get("~lyricsdescription", ""),
+                                     lang=self["~lyricslanguage"]))
 
         # Delete old foobar replaygain ..
         for frame in tag.getall("TXXX"):
@@ -392,7 +394,7 @@ class ID3File(AudioFile):
                 value = self[k]
                 tag.add(mutagen.id3.TXXX(encoding=encoding_for(value),
                                          text=value.split("\n"),
-                                         desc=k))
+                                         desc=k.upper()))
 
         # we shouldn't delete all, but we use unknown ones as fallback, so make
         # sure they don't come back after reloading
@@ -404,7 +406,7 @@ class ID3File(AudioFile):
             if ('replaygain_%s_gain' % k) in self:
                 try:
                     gain = float(self["replaygain_%s_gain" % k].split()[0])
-                except ValueError:
+                except (ValueError, IndexError):
                     gain = 0
                 try:
                     peak = float(self["replaygain_%s_peak" % k])
@@ -453,9 +455,9 @@ class ID3File(AudioFile):
         with translate_errors():
             audio = self.Kind(self["~filename"])
 
-        if audio.tags is not None:
-            audio.tags.delall("APIC")
-            audio.save()
+            if audio.tags is not None:
+                audio.tags.delall("APIC")
+                audio.save()
 
         self.has_images = False
 
