@@ -1,8 +1,9 @@
 # Copyright 2006 Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 """Various library classes.
 
@@ -13,14 +14,12 @@ also be queried in various ways.
 """
 
 import time
-import threading
 
-from quodlibet import util, print_d
-import quodlibet.formats as formats
-from quodlibet.const import LIBRARY_SAVE_PERIOD_SECONDS
+from quodlibet import print_d
 
-from quodlibet.library.libraries import SongFileLibrary, SongLibrary
+from quodlibet.library.song import SongLibrary, SongFileLibrary
 from quodlibet.library.librarians import SongLibrarian
+from quodlibet.util.path import mtime
 
 
 def init(cache_fn=None):
@@ -29,8 +28,7 @@ def init(cache_fn=None):
     Return a main library, and set a librarian for
     all future SongLibraries.
     """
-    s = ", ".join(formats.modules)
-    print_d("Supported formats: %s" % s)
+
     SongFileLibrary.librarian = SongLibrary.librarian = SongLibrarian()
     library = SongFileLibrary("main")
     if cache_fn:
@@ -38,11 +36,11 @@ def init(cache_fn=None):
     return library
 
 
-def save(force=False):
+def save(save_period=None):
     """Save all registered libraries that have a filename and are marked dirty.
 
-    If force = True save all of them blocking, else save non-blocking and
-    only if they were last saved more than LIBRARY_SAVE_PERIOD_SECONDS ago.
+    If `save_period` (seconds) is given the library will only be saved if
+    it hasn't been in the last `save_period` seconds.
     """
 
     print_d("Saving all libraries...")
@@ -53,11 +51,5 @@ def save(force=False):
         if not filename or not lib.dirty:
             continue
 
-        if force:
-            try:
-                lib.save()
-            except EnvironmentError:
-                pass
-            lib.destroy()
-        elif time.time() - util.mtime(filename) > LIBRARY_SAVE_PERIOD_SECONDS:
-            threading.Thread(target=lib.save).run()
+        if not save_period or abs(time.time() - mtime(filename)) > save_period:
+            lib.save()
