@@ -5,6 +5,8 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 
+from typing import List
+
 from gi.repository import Gtk
 
 from quodlibet.util import connect_obj
@@ -12,7 +14,7 @@ from quodlibet.util.misc import total_ordering
 
 
 @total_ordering
-class RenameFilesPlugin(object):
+class RenameFilesPlugin:
     """Plugins of this type must subclass a GTK widget. They will be
     packed into the RenameFiles pane (currently a ScrolledWindow hidden
     with an expander, but that might change).
@@ -33,7 +35,7 @@ class RenameFilesPlugin(object):
     default filters have orders between 1 and 2. Plugins have order 0 by
     default. Plugins with equal orders are sorted by class name."""
 
-    _order = 0.0
+    _order = 0
     active = False
 
     def filter(self, original_filename, value):
@@ -52,7 +54,7 @@ class RenameFilesPlugin(object):
 
 
 @total_ordering
-class TagsFromPathPlugin(object):
+class TagsFromPathPlugin:
     """Plugins of this type must subclass a GTK widget. They will be
     packed into the TagsFromPath pane (currently a ScrolledWindow hidden
     with an expander, but that might change).
@@ -131,8 +133,8 @@ class EditTagsPlugin(Gtk.ImageMenuItem):
          clicked on, and your activate handler runs.
     """
 
-    tags = []
-    needs = []
+    tags: List[str] = []
+    needs: List[str] = []
     _order = 2.0
 
     def activated(self, tag, value):
@@ -143,5 +145,5 @@ class EditTagsPlugin(Gtk.ImageMenuItem):
             for item in self.get_submenu().get_children():
                 connect_obj(item, signal, callback, self, *args, **kwargs)
         else:
-            super(EditTagsPlugin, self).connect(
+            super().connect(
                 signal, callback, *args, **kwargs)
