@@ -1,23 +1,26 @@
-# -*- coding: utf-8 -*-
 # Copyright 2004-2006 Joe Wreschnig, Michael Urman, Iñigo Serna
 #                2014 Nick Boultbee
-#                2014 Nick Boultbee
+#                2017 Fredrik Strupe
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+from typing import Optional, Type
 
 from gi.repository import Gtk, GObject
+from senf import fsn2text
 
 from quodlibet import config
 from quodlibet import util
-
+from quodlibet import _
 from quodlibet.plugins import PluginHandler
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.msg import WarningMessage, ErrorMessage
 from quodlibet.qltk import Icons
-from quodlibet.util.path import fsdecode
-from quodlibet.util import connect_obj
+from quodlibet.util import connect_obj, connect_destroy
+from quodlibet.errorreport import errorhook
 
 
 class OverwriteWarning(WarningMessage):
@@ -27,12 +30,12 @@ class OverwriteWarning(WarningMessage):
     def __init__(self, parent, song):
         title = _("Tag may not be accurate")
 
-        fn_format = "<b>%s</b>" % util.escape(fsdecode(song("~basename")))
+        fn_format = "<b>%s</b>" % util.escape(fsn2text(song("~basename")))
         description = _("%(file-name)s changed while the program was running. "
             "Saving without refreshing your library may "
             "overwrite other changes to the song.") % {"file-name": fn_format}
 
-        super(OverwriteWarning, self).__init__(
+        super().__init__(
             parent, title, description, buttons=Gtk.ButtonsType.NONE)
 
         self.add_button(_("_Cancel"), Gtk.ResponseType.CANCEL)
@@ -46,12 +49,12 @@ class WriteFailedError(ErrorMessage):
     def __init__(self, parent, song):
         title = _("Unable to save song")
 
-        fn_format = "<b>%s</b>" % util.escape(fsdecode(song("~basename")))
+        fn_format = "<b>%s</b>" % util.escape(fsn2text(song("~basename")))
         description = _("Saving %(file-name)s failed. The file may be "
             "read-only, corrupted, or you do not have "
             "permission to edit it.") % {"file-name": fn_format}
 
-        super(WriteFailedError, self).__init__(
+        super().__init__(
             parent, title, description)
 
 
@@ -60,10 +63,10 @@ class EditingPluginHandler(GObject.GObject, PluginHandler):
         "changed": (GObject.SignalFlags.RUN_LAST, None, ())
     }
 
-    Kind = None
+    Kind: Optional[Type] = None
 
     def __init__(self):
-        super(EditingPluginHandler, self).__init__()
+        super().__init__()
         self.__plugins = []
 
     @property
@@ -89,10 +92,11 @@ class FilterCheckButton(ConfigCheckButton):
     __gsignals__ = {
         "preview": (GObject.SignalFlags.RUN_LAST, None, ())
         }
+    _tooltip = None
 
     def __init__(self):
-        super(FilterCheckButton, self).__init__(
-            self._label, self._section, self._key)
+        super().__init__(
+            self._label, self._section, self._key, tooltip=self._tooltip)
         try:
             self.set_active(config.getboolean(self._section, self._key))
         except:
@@ -107,7 +111,7 @@ class FilterCheckButton(ConfigCheckButton):
         raise NotImplementedError
 
     def filter_list(self, origs, names):
-        return map(self.filter, origs, names)
+        return list(map(self.filter, origs, names))
 
     def __lt__(self, other):
         return (self._order, type(self).__name__) < \
@@ -124,7 +128,7 @@ class FilterPluginBox(Gtk.VBox):
     }
 
     def __init__(self, plugin_handler, filter_types=None):
-        super(FilterPluginBox, self).__init__()
+        super().__init__()
 
         # static filters
         if filter_types is None:
@@ -141,32 +145,26 @@ class FilterPluginBox(Gtk.VBox):
         hb = Gtk.HBox()
         expander = Gtk.Expander(label=_(u"_More options…"))
         expander.set_use_underline(True)
+        expander.set_no_show_all(True)
         hb.pack_start(expander, True, True, 0)
         self.pack_start(hb, False, True, 0)
 
         for filt in filters:
             filt.connect('preview', lambda *x: self.emit("preview"))
 
-        sw = Gtk.ScrolledWindow()
-        sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
-
         vbox = Gtk.VBox()
+        expander.add(vbox)
 
-        plugin_handler.connect(
+        connect_destroy(plugin_handler,
             "changed", self.__refresh_plugins, vbox, expander)
 
-        sw.add_with_viewport(vbox)
-        self.pack_start(sw, False, True, 0)
-
-        sw.set_no_show_all(True)
-        expander.connect("notify::expanded", self.__notify_expanded, sw)
+        expander.connect("notify::expanded", self.__notify_expanded, vbox)
         expander.set_expanded(False)
 
         for child in self.get_children():
             child.show()
 
         plugin_handler.changed()
-        sw.hide()
 
     def __notify_expanded(self, expander, event, vbox):
         vbox.set_property('visible', expander.get_property('expanded'))
@@ -177,7 +175,7 @@ class FilterPluginBox(Gtk.VBox):
             try:
                 f = Kind()
             except:
-                util.print_exc()
+                errorhook()
                 continue
             else:
                 instances.append(f)
@@ -191,7 +189,7 @@ class FilterPluginBox(Gtk.VBox):
             try:
                 vbox.pack_start(f, True, True, 0)
             except:
-                util.print_exc()
+                errorhook()
                 f.destroy()
                 continue
 
@@ -201,7 +199,7 @@ class FilterPluginBox(Gtk.VBox):
                 try:
                     f.connect('changed', lambda *x: self.emit('changed'))
                 except:
-                    util.print_exc()
+                    errorhook()
                     continue
 
             self.__plugins.append(f)
