@@ -9,7 +9,7 @@
 
 from gi.repository import Gtk, Pango
 
-from quodlibet import config
+from quodlibet import config, print_d
 from quodlibet import util
 from quodlibet import browsers
 from quodlibet import app
@@ -315,6 +315,7 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
             bg = background_filter()
             if bg:
                 songs = list(filter(bg, songs))
+        print_d(f"Setting {len(songs)} songs...")
         self.songlist.set_songs(songs, sorted)
 
     def __enqueue(self, view, path, column, player):
@@ -349,6 +350,5 @@ class LibraryBrowser(Window, util.InstanceTracker, PersistentWindowMixin):
     def __set_totals(self, info, songs):
         i = len(songs)
         length = sum(song.get("~#length", 0) for song in songs)
-        t = self.browser.status_text(count=i,
-                                     time=util.format_time_preferred(length))
+        t = self.browser.status_text(count=i, time=util.format_time_preferred(length))
         self.__statusbar.set_text(t)
