@@ -33,7 +33,7 @@ class JSONBasedEditor(qltk.UniqueWindow):
     def __init__(self, Prototype, values, filename, title):
         if self.is_not_unique():
             return
-        super(JSONBasedEditor, self).__init__()
+        super().__init__()
         self.Prototype = Prototype
         self.current = None
         self.filename = filename
@@ -198,7 +198,7 @@ class JSONBasedEditor(qltk.UniqueWindow):
             l.set_mnemonic_widget(entry)
             l.set_use_underline(True)
             l.set_alignment(0.0, 0.5)
-            if isinstance(val, int) or isinstance(val, bool):
+            if isinstance(val, (int, bool)):
                 align = Align(entry, halign=Gtk.Align.START)
                 t.attach(align, 1, 2, i, i + 1)
             else:
@@ -265,7 +265,7 @@ class TagListEditor(qltk.Window):
     _HEIGHT = 300
 
     def __init__(self, title, values=None):
-        super(TagListEditor, self).__init__()
+        super().__init__()
         self.use_header_bar()
         self.data = values or []
         self.set_border_width(12)
