@@ -1,84 +1,75 @@
-# -*- coding: utf-8 -*-
 # Copyright 2006 Joe Wreschnig
-#      2013-2014 Nick Boultbee
+#      2013-2022 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 from gi.repository import Gtk
 
-from quodlibet import qltk
+from quodlibet.formats import AudioFile
+from quodlibet.plugins.gui import MenuItemPlugin
+from quodlibet.plugins.songshelpers import is_a_file
+from quodlibet.qltk.chooser import choose_folders
+from quodlibet.qltk.download import DownloadProgress
+from quodlibet.qltk.pluginwin import PluginWindow
 
-from quodlibet.util import print_exc
-from quodlibet.qltk.msg import WarningMessage
+from quodlibet import ngettext, _, print_d, app
+from quodlibet import qltk
+from quodlibet.errorreport import errorhook
+from quodlibet.qltk.showfiles import show_songs
+
+from quodlibet.util import print_e, print_w, copool
+from quodlibet.qltk.msg import ConfirmationPrompt, ErrorMessage, Message
 from quodlibet.qltk.delete import TrashMenuItem, trash_songs
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.properties import SongProperties
-from quodlibet.qltk.x import SeparatorMenuItem, Button
+from quodlibet.qltk.x import SeparatorMenuItem
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
-from quodlibet.qltk import get_top_parent, get_menu_item_top_parent
+from quodlibet.qltk import get_top_parent, get_menu_item_top_parent, Icons
 from quodlibet.plugins import PluginManager, PluginHandler
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.songwrapper import ListWrapper, check_wrapper_changed
 
 
-class ConfirmMultiSongInvoke(WarningMessage):
+def confirm_song_removal_invoke(parent, songs):
+    songs = set(songs)
+    if not songs:
+        return True
+
+    count = len(songs)
+    song = next(iter(songs))
+    title = (ngettext("Remove track: \"%%(title)s\" from library?",
+                      "Remove %(count)d tracks from library?",
+                      count) % {'count': count}) % {
+                'title': song('title') or song('~basename')}
+
+    prompt = ConfirmationPrompt(parent, title, "", _("Remove from Library"),
+                                ok_button_icon=Icons.LIST_REMOVE)
+    return prompt.run() == ConfirmationPrompt.RESPONSE_INVOKE
+
+
+def confirm_multi_song_invoke(parent, plugin_name, count):
     """Dialog to confirm invoking a plugin with X songs in case X is high"""
-
-    RESPONSE_INVOKE = 1
-
-    def __init__(self, parent, plugin_name, count):
-        title = ngettext("Run the plugin \"%(name)s\" on %(count)d song?",
-                         "Run the plugin \"%(name)s\" on %(count)d songs?",
-                         count) % {"name": plugin_name, "count": count}
-
-        super(ConfirmMultiSongInvoke, self).__init__(
-            get_top_parent(parent),
-            title, "",
-            buttons=Gtk.ButtonsType.NONE)
-
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        delete_button = Button(_("_Run Plugin"), Gtk.STOCK_EXECUTE)
-        delete_button.show()
-        self.add_action_widget(delete_button, self.RESPONSE_INVOKE)
-        self.set_default_response(Gtk.ResponseType.CANCEL)
-
-    @classmethod
-    def confirm(cls, parent, plugin_name, count):
-        """Returns if the action was confirmed"""
-
-        resp = cls(parent, plugin_name, count).run()
-        return resp == cls.RESPONSE_INVOKE
+    title = ngettext("Run the plugin \"%(name)s\" on %(count)d song?",
+                     "Run the plugin \"%(name)s\" on %(count)d songs?",
+                     count) % {"name": plugin_name, "count": count}
+    description = ""
+    ok_text = _("_Run Plugin")
+    prompt = ConfirmationPrompt(parent, title, description, ok_text).run()
+    return prompt == ConfirmationPrompt.RESPONSE_INVOKE
 
 
-class ConfirmMultiAlbumInvoke(WarningMessage):
+def confirm_multi_album_invoke(parent, plugin_name, count):
     """Dialog to confirm invoking a plugin with X albums in case X is high"""
-
-    RESPONSE_INVOKE = 1
-
-    def __init__(self, parent, plugin_name, count):
-        title = ngettext("Run the plugin \"%(name)s\" on %(count)d album?",
-                         "Run the plugin \"%(name)s\" on %(count)d albums?",
-                         count) % {'name': plugin_name, 'count': count}
-
-        super(ConfirmMultiAlbumInvoke, self).__init__(
-            get_top_parent(parent),
-            title, "",
-            buttons=Gtk.ButtonsType.NONE)
-
-        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        delete_button = Button(_("_Run Plugin"), Gtk.STOCK_EXECUTE)
-        delete_button.show()
-        self.add_action_widget(delete_button, self.RESPONSE_INVOKE)
-        self.set_default_response(Gtk.ResponseType.CANCEL)
-
-    @classmethod
-    def confirm(cls, parent, plugin_name, count):
-        """Returns if the action was confirmed"""
-
-        resp = cls(parent, plugin_name, count).run()
-        return resp == cls.RESPONSE_INVOKE
+    title = ngettext("Run the plugin \"%(name)s\" on %(count)d album?",
+                     "Run the plugin \"%(name)s\" on %(count)d albums?",
+                     count) % {"name": plugin_name, "count": count}
+    description = ""
+    ok_text = _("_Run Plugin")
+    prompt = ConfirmationPrompt(parent, title, description, ok_text).run()
+    return prompt == ConfirmationPrompt.RESPONSE_INVOKE
 
 
 class SongsMenuPluginHandler(PluginHandler):
@@ -88,11 +79,11 @@ class SongsMenuPluginHandler(PluginHandler):
 
         self.__plugins = []
 
-        self._confirm_multiple_songs = ConfirmMultiSongInvoke.confirm
+        self._confirm_multiple_songs = confirm_multi_song_invoke
         if song_confirmer is not None:
             self._confirm_multiple_songs = song_confirmer
 
-        self._confirm_multiple_albums = ConfirmMultiAlbumInvoke.confirm
+        self._confirm_multiple_albums = confirm_multi_album_invoke
         if album_confirmer is not None:
             self._confirm_multiple_albums = album_confirmer
 
@@ -117,15 +108,15 @@ class SongsMenuPluginHandler(PluginHandler):
         kinds = self.__plugins
         kinds.sort(key=lambda plugin: plugin.PLUGIN_ID)
         for Kind in kinds:
-            usable = max([callable(getattr(Kind, s)) for s in attrs])
+            usable = any(callable(getattr(Kind, s)) for s in attrs)
             if usable:
                 try:
                     items.append(Kind(songs, library))
                 except:
                     print_e("Couldn't initialise song plugin %s. Stack trace:"
                             % Kind)
-                    print_exc()
-        items = filter(lambda i: i.initialized, items)
+                    errorhook()
+        items = [i for i in items if i.initialized]
 
         if items:
             menu = Gtk.Menu()
@@ -141,8 +132,12 @@ class SongsMenuPluginHandler(PluginHandler):
                         item.connect(
                             'activate', self.__on_activate, item, *args)
                 except:
-                    print_exc()
+                    errorhook()
                     item.destroy()
+            menu.append(SeparatorMenuItem())
+            prefs = Gtk.MenuItem(label=_("Configure Plugins…"))
+            prefs.connect("activate", lambda _: PluginWindow().show())
+            menu.append(prefs)
 
         else:
             menu = None
@@ -156,7 +151,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 albums[key] = []
             albums[key].append(song)
 
-        albums = albums.values()
+        albums = list(albums.values())
         for album in albums:
             album.sort()
         return albums
@@ -172,7 +167,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 try:
                     plugin = plugin(songs, library)
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
                     self.__handle(plugin, plugin, library, songs, parent)
                 return
@@ -190,7 +185,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 try:
                     ret = plugin.plugin_single_song(songs[0])
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
                     if ret:
                         return
@@ -204,15 +199,15 @@ class SongsMenuPluginHandler(PluginHandler):
                 try:
                     ret = map(plugin.plugin_song, songs)
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
-                    if max(ret):
+                    if any(ret):
                         return
             if callable(plugin.plugin_songs):
                 try:
                     ret = plugin.plugin_songs(songs)
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
                     if ret:
                         return
@@ -229,7 +224,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 try:
                     ret = plugin.plugin_single_album(albums[0])
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
                     if ret:
                         return
@@ -237,15 +232,15 @@ class SongsMenuPluginHandler(PluginHandler):
                 try:
                     ret = map(plugin.plugin_album, albums)
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
-                    if max(ret):
+                    if any(ret):
                         return
             if callable(plugin.plugin_albums):
                 try:
                     ret = plugin.plugin_albums(albums)
                 except Exception:
-                    print_exc()
+                    errorhook()
                 else:
                     if ret:
                         return
@@ -270,11 +265,11 @@ class SongsMenu(Gtk.Menu):
     def init_plugins(cls):
         PluginManager.instance.register_handler(cls.plugins)
 
-    def __init__(self, library, songs, plugins=True, playlists=True,
-                 queue=True, devices=True, remove=True, delete=False,
-                 edit=True, ratings=True, items=None, accels=True):
-        super(SongsMenu, self).__init__()
-
+    def __init__(self, library, songs, plugins=True, playlists=True, queue=True,
+                 remove=True, delete=False, edit=True, ratings=True, show_files=True,
+                 download=False, items=None, accels=True,
+                 removal_confirmer=None, folder_chooser=None):
+        super().__init__()
         # The library may actually be a librarian; if it is, use it,
         # otherwise find the real librarian.
         librarian = getattr(library, 'librarian', library)
@@ -295,7 +290,7 @@ class SongsMenu(Gtk.Menu):
         if plugins:
             submenu = self.plugins.Menu(librarian, songs)
             if submenu is not None:
-                b = qltk.MenuItem(_("_Plugins"), Gtk.STOCK_EXECUTE)
+                b = qltk.MenuItem(_("_Plugins"), Icons.SYSTEM_RUN)
                 b.set_sensitive(bool(songs))
                 self.append(b)
                 b.set_submenu(submenu)
@@ -313,62 +308,45 @@ class SongsMenu(Gtk.Menu):
                 is_file = False
 
         if playlists:
-            # Needed here to avoid a circular import; most browsers use
-            # a SongsMenu, but SongsMenu needs access to the playlist
-            # browser for this item.
-
-            # FIXME: Two things are now importing browsers, so we need
-            # some kind of inversion of control here.
-            from quodlibet.browsers.playlists.menu import PlaylistMenu
             try:
-                submenu = PlaylistMenu(songs)
+                from quodlibet.browsers.playlists.menu import PlaylistMenu
+                submenu = PlaylistMenu(songs, library.playlists)
             except AttributeError as e:
                 print_w("Couldn't get Playlists menu: %s" % e)
             else:
-                b = qltk.MenuItem(_("Play_lists"), Gtk.STOCK_ADD)
+                b = qltk.MenuItem(_("Play_lists"), Icons.FOLDER_DRAG_ACCEPT)
                 b.set_sensitive(can_add and bool(songs))
                 b.set_submenu(submenu)
                 self.append(b)
         if queue:
-            b = qltk.MenuItem(_("Add to _Queue"), Gtk.STOCK_ADD)
+            b = qltk.MenuItem(_("Add to _Queue"), Icons.LIST_ADD)
 
             def enqueue_cb(item, songs):
-                songs = filter(lambda s: s.can_add, songs)
+                songs = [s for s in songs if s.can_add]
                 if songs:
                     from quodlibet import app
                     app.window.playlist.enqueue(songs)
 
             b.connect('activate', enqueue_cb, songs)
             if accels:
-                qltk.add_fake_accel(b, "<ctrl>Return")
+                qltk.add_fake_accel(b, "<Primary>Return")
             self.append(b)
             b.set_sensitive(can_add and bool(songs))
 
-        if devices:
-            from quodlibet import browsers
-            try:
-                browsers.media
-            except AttributeError:
-                pass
-            else:
-                if browsers.media.MediaDevices in browsers.browsers:
-                    submenu = browsers.media.Menu(songs, library)
-                    b = qltk.MenuItem(_("_Copy to Device"), Gtk.STOCK_COPY)
-                    b.set_sensitive(
-                        can_add and len(submenu) > 0 and bool(songs))
-                    b.set_submenu(submenu)
-                    self.append(b)
-
         if remove or delete:
             self.separate()
 
         if remove:
-            b = qltk.MenuItem(_("_Remove from library"), Gtk.STOCK_REMOVE)
+            self._confirm_song_removal = (removal_confirmer or
+                                          confirm_song_removal_invoke)
+            b = qltk.MenuItem(_("_Remove from Library…"), Icons.LIST_REMOVE)
             if callable(remove):
                 b.connect('activate', lambda item: remove(songs))
             else:
                 def remove_cb(item, songs, library):
-                    library.remove(set(songs))
+                    parent = get_menu_item_top_parent(item)
+                    if self._confirm_song_removal(parent, songs):
+                        library.remove(songs)
 
                 b.connect('activate', remove_cb, songs, library)
                 b.set_sensitive(in_lib and bool(songs))
@@ -376,10 +354,14 @@ class SongsMenu(Gtk.Menu):
 
         if delete:
             if callable(delete):
-                b = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
+                b = qltk.MenuItem(_("_Delete"), Icons.EDIT_DELETE)
                 b.connect('activate', lambda item: delete(songs))
+                if accels:
+                    qltk.add_fake_accel(b, "<Primary>Delete")
             else:
                 b = TrashMenuItem()
+                if accels:
+                    qltk.add_fake_accel(b, "<Primary>Delete")
 
                 def trash_cb(item):
                     parent = get_menu_item_top_parent(item)
@@ -391,7 +373,7 @@ class SongsMenu(Gtk.Menu):
 
         if edit:
             self.separate()
-            b = qltk.MenuItem(_("Edit _Tags"), Gtk.STOCK_PROPERTIES)
+            b = qltk.MenuItem(_("Edit _Tags"), Icons.EDIT)
             b.set_sensitive(bool(songs))
             if accels:
                 qltk.add_fake_accel(b, "<alt>Return")
@@ -404,18 +386,86 @@ class SongsMenu(Gtk.Menu):
             b.connect('activate', song_properties_cb)
             self.append(b)
 
-            b = Gtk.ImageMenuItem(label=Gtk.STOCK_INFO, use_stock=True)
+            b = qltk.MenuItem(_("_Information"), Icons.DIALOG_INFORMATION)
             b.set_sensitive(bool(songs))
             if accels:
-                qltk.add_fake_accel(b, "<ctrl>I")
+                qltk.add_fake_accel(b, "<Primary>I")
 
             def information_cb(menu_item):
                 parent = get_menu_item_top_parent(menu_item)
                 window = Information(librarian, songs, parent)
                 window.show()
+
             b.connect('activate', information_cb)
             self.append(b)
 
+        if show_files and any(is_a_file(s) for s in songs):
+            def show_files_cb(menu_item):
+                print_d("Trying to show files...")
+                if not show_songs(songs):
+                    parent = get_menu_item_top_parent(menu_item)
+                    msg = ErrorMessage(parent,
+                                 _("Unable to show files"),
+                                 _("Error showing files, "
+                                   "or no program available to show them."))
+                    msg.run()
+
+            self.separate()
+            total = len([s for s in songs if is_a_file(s)])
+            text = ngettext(
+                "_Show in File Manager",
+                "_Show %(total)d Files in File Manager", total) % {"total": total}
+            b = qltk.MenuItem(text, Icons.DOCUMENT_OPEN)
+            b.set_sensitive(bool(songs)
+                            and len(songs) < MenuItemPlugin.MAX_INVOCATIONS)
+            b.connect('activate', show_files_cb)
+            self.append(b)
+
+        if download:
+
+            def is_downloadable(song: AudioFile):
+                return bool(not song.is_file and song.get("~uri", False))
+
+            self.separate()
+            relevant = [s for s in songs if is_downloadable(s)]
+            total = len(relevant)
+            text = ngettext(
+                "_Download file…",
+                "_Download %(total)d files…", total) % {"total": total}
+            b = qltk.MenuItem(text, Icons.EMBLEM_DOWNLOADS)
+            b.set_sensitive(relevant
+                            and len(relevant) < MenuItemPlugin.MAX_INVOCATIONS)
+
+            def _finished(p, successes, failures):
+                msg = (f"<b>{successes}</b> " + _("successful") +
+                       f"\n<b>{failures}</b> " + _("failed"))
+                print_d(msg.replace("\n", "; "))
+                warning = Message(Gtk.MessageType.INFO, app.window,
+                                  _("Downloads complete"), msg, escape_desc=False)
+                warning.run()
+
+            def download_cb(menu_item):
+                songs = relevant
+                total = len(songs)
+                msg = ngettext("Download {name!r} to",
+                               "Download {total} files to",
+                               total)
+                msg = msg.format(name=next(iter(songs))("title")[:99] if total else "?",
+                                 total=total)
+                chooser = folder_chooser or choose_folders
+                paths = chooser(None, msg, _("Download here"), allow_multiple=False)
+                if not paths:
+                    print_d("Cancelling download")
+                    return
+                path = paths[0]
+                progress = DownloadProgress(songs)
+
+                progress.connect('finished', _finished)
+                copool.add(progress.download_songs, path)
+
+            b.connect('activate', download_cb)
+            self.append(b)
+
         def selection_done_cb(menu):
             menu.destroy()
 
