@@ -7,6 +7,7 @@
 # (at your option) any later version.
 
 import contextlib
+import sys
 
 from gi.repository import Gtk, Gdk, GObject, Pango, GLib
 import cairo
@@ -18,6 +19,8 @@ from quodlibet.qltk import get_top_parent, is_accel, is_wayland, gtk_version, \
     menu_popup, get_primary_accel_mod
 from quodlibet.qltk.image import get_surface_extents
 
+from .util import GSignals
+
 
 class TreeViewHints(Gtk.Window):
     """Handle 'hints' for treeviews. This includes expansions of truncated
@@ -31,12 +34,14 @@ class TreeViewHints(Gtk.Window):
     # input_shape_combine_region does not work under Windows, we have
     # to pass all events to the treeview. In case it does work, this handlers
     # will never be called.
-    __gsignals__ = dict.fromkeys(
+    __gsignals__: GSignals = dict.fromkeys(
         ['button-press-event', 'button-release-event',
          'motion-notify-event', 'scroll-event',
          'enter-notify-event', 'leave-notify-event'],
         'override')
 
+    __empty_region = cairo.Region(cairo.RectangleInt())
+
     def __init__(self):
         try:
             # gtk+ 3.20
@@ -44,7 +49,7 @@ class TreeViewHints(Gtk.Window):
         except AttributeError:
             pass
 
-        super(TreeViewHints, self).__init__(type=Gtk.WindowType.POPUP)
+        super().__init__(type=Gtk.WindowType.POPUP)
         # set the type hint so the wayland backend maps it as a subsurface
         # which supports relative positioning
         self.set_type_hint(Gdk.WindowTypeHint.TOOLTIP)
@@ -349,6 +354,13 @@ class TreeViewHints(Gtk.Window):
         set_text(label)
         self.set_size_request(w, h)
 
+        # Set region on this window for which to receive mouse events to the
+        # empty region. Mouse events will be passed to the window below the
+        # tooltip. The Gdk implementation for win32 does not support this, which
+        # leads to events not being received in either window.
+        if sys.platform != "win32":
+            self.input_shape_combine_region(self.__empty_region)
+
         window = self.get_window()
         if self.get_visible() and window:
             window.move_resize(x, y, w, h)
@@ -488,7 +500,7 @@ class TreeViewHints(Gtk.Window):
         return self.__event(event)
 
 
-class DragScroll(object):
+class DragScroll:
     """A treeview mixin for smooth drag and scroll (needs BaseView).
 
     Call scroll_motion in the 'drag-motion' handler and
@@ -613,15 +625,14 @@ class DragScroll(object):
 
 class BaseView(Gtk.TreeView):
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         # like the tree selection changed signal but doesn't emit twice in case
         # a row is activated
-        'selection-changed': (
-            GObject.SignalFlags.RUN_LAST, None, (object, )),
+        'selection-changed': (GObject.SignalFlags.RUN_LAST, None, (object, )),
     }
 
     def __init__(self, *args, **kwargs):
-        super(BaseView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connect("key-press-event", self.__key_pressed)
         self._setup_selection_signal()
 
@@ -853,7 +864,7 @@ class BaseView(Gtk.TreeView):
 
     @contextlib.contextmanager
     def without_model(self):
-        """Conext manager which removes the model from the view
+        """Context manager which removes the model from the view
         and adds it back afterwards.
 
         Tries to preserve all state that gets reset on a model change.
@@ -879,7 +890,7 @@ class DragIconTreeView(BaseView):
     """
 
     def __init__(self, *args, **kwargs):
-        super(DragIconTreeView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connect('drag-begin', self.__begin)
 
     def __begin(self, view, drag_ctx):
@@ -974,7 +985,7 @@ class MultiDragTreeView(BaseView):
     """
 
     def __init__(self, *args, **kwargs):
-        super(MultiDragTreeView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connect('button-press-event', self.__button_press)
         self.connect('button-release-event', self.__button_release)
         self.__pending_action = None
@@ -1017,7 +1028,7 @@ class RCMTreeView(BaseView):
     """Emits popup-menu when a row is right-clicked on."""
 
     def __init__(self, *args, **kwargs):
-        super(RCMTreeView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.connect('button-press-event', self.__button_press)
 
     def __button_press(self, view, event):
@@ -1124,7 +1135,7 @@ class HintedTreeView(BaseView):
     contains ellipsized text."""
 
     def __init__(self, *args, **kwargs):
-        super(HintedTreeView, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.supports_hints():
             try:
                 tvh = HintedTreeView.hints
@@ -1135,7 +1146,7 @@ class HintedTreeView(BaseView):
     def set_tooltip_text(self, *args, **kwargs):
         print_e("Setting a tooltip on the view breaks tv hints. Set it"
                 " on the parent scrolled window instead")
-        return super(HintedTreeView, self).set_tooltip_text(*args, **kwargs)
+        return super().set_tooltip_text(*args, **kwargs)
 
     def supports_hints(self):
         """If the treeview hints support is enabled. Can be used to
@@ -1224,7 +1235,7 @@ class _TreeViewColumnLabel(Gtk.Label):
 
 class TreeViewColumn(Gtk.TreeViewColumn):
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         # tree-view-changed(old_tree_view, new_tree_view)
         # Triggers when the columns gets added/removed from a tree view.
         # The passed values are either a TreeView or None
@@ -1277,13 +1288,13 @@ class TreeViewColumnButton(TreeViewColumn):
     """A TreeViewColumn that forwards its header events:
         button-press-event and popup-menu"""
 
-    __gsignals__ = {
+    __gsignals__: GSignals = {
         'button-press-event': (GObject.SignalFlags.RUN_LAST, bool, (object,)),
-        'popup-menu': (GObject.SignalFlags.RUN_LAST, bool, ()),
+        'popup-menu': (GObject.SignalFlags.RUN_LAST, bool, tuple()),
     }
 
     def __init__(self, **kwargs):
-        super(TreeViewColumnButton, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         label = self.get_widget()
         label.__realize = label.connect('realize', self.__connect_menu_event)
 
