@@ -110,7 +110,7 @@ def init_fake_app():
 
     from quodlibet import browsers
     from quodlibet.player.nullbe import NullPlayer
-    from quodlibet.library.libraries import SongFileLibrary
+    from quodlibet.library import SongFileLibrary
     from quodlibet.library.librarians import SongLibrarian
     from quodlibet.qltk.quodlibetwindow import QuodLibetWindow, PlayerOptions
     from quodlibet.util.cover import CoverManager
@@ -196,6 +196,11 @@ def init_test_environ():
     # despite this setting.
     environ["GST_REGISTRY_UPDATE"] = fsnative(u"no")
 
+    # In flatpak we might get a registry from a different/old flatpak build
+    # when testing new versions etc. Better always update in that case.
+    if util.is_flatpak():
+        del environ["GST_REGISTRY_UPDATE"]
+
     # set HOME and remove all XDG vars that default to it if not set
     home_dir = tempfile.mkdtemp(prefix=fsnative(u"HOME-"), dir=_TEMP_DIR)
     environ["HOME"] = home_dir
@@ -304,3 +309,12 @@ def unit(run=[], suite=None, strict=False, exitfirst=False, network=True,
         args.append(os.path.join("tests", suite))
 
     return pytest.main(args=args)
+
+
+def run_gtk_loop():
+    """Exhausts the GTK main loop of any events"""
+
+    # Import late as various version / init checks fail otherwise
+    from gi.repository import Gtk
+    while Gtk.events_pending():
+        Gtk.main_iteration()
