@@ -2,19 +2,16 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
-from tests import TestCase
-
-from gi.repository import Gtk
-from senf import fsnative
+import os
 
 import quodlibet.browsers.tracks
 import quodlibet.config
-
 from quodlibet.browsers.tracks import TrackList
 from quodlibet.formats import AudioFile
 from quodlibet.library import SongLibrary, SongLibrarian
 from quodlibet.qltk.songlist import SongList
+from senf import fsnative
+from tests import TestCase, run_gtk_loop
 
 # Don't sort yet, album_key makes it complicated...
 SONGS = [AudioFile({
@@ -55,7 +52,7 @@ class TSearchBar(TestCase):
             af.sanitize()
         quodlibet.browsers.tracks.library.add(SONGS)
         self.bar = self.Bar(quodlibet.browsers.tracks.library)
-        self.bar.connect('songs-selected', self._expected)
+        self._sid = self.bar.connect('songs-selected', self._expected)
         self.success = False
 
     def _expected(self, bar, songs, sort):
@@ -64,8 +61,7 @@ class TSearchBar(TestCase):
         self.success = True
 
     def _do(self):
-        while Gtk.events_pending():
-            Gtk.main_iteration()
+        run_gtk_loop()
         self.failUnless(self.success or self.expected is None)
 
     def test_can_filter(self):
@@ -145,6 +141,12 @@ class TSearchBar(TestCase):
         self._do()
 
     def tearDown(self):
+        for song in SONGS:
+            try:
+                os.unlink(song("~filename"))
+            except OSError:
+                pass
+        self.bar.disconnect(self._sid)
         self.bar.destroy()
         quodlibet.browsers.tracks.library.destroy()
         quodlibet.config.quit()
