@@ -10,14 +10,13 @@ from quodlibet import plugins
 from quodlibet import config
 
 from quodlibet.plugins import Plugin
-from quodlibet.plugins.cover import CoverSourcePlugin
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk.pluginwin import PluginWindow, PluginErrorWindow, \
     PluginListView, PluginEnabledFilterCombo, PluginPreferencesContainer, \
     EnabledType, PluginTypeFilterCombo
 
 
-class FakePlugin(object):
+class FakePlugin:
     PLUGIN_ID = "fo<o"
     PLUGIN_NAME = "b>ar"
     PLUGIN_DESC = "quux"
@@ -69,10 +68,10 @@ class TPluginWindow(TestCase):
     def test_type_filter_combo(self):
         combo = PluginTypeFilterCombo()
         # The ALL item should be first.
-        self.failUnlessEqual(combo.get_active_type(), object)
-        # Separator counts as one too
-        combo.set_active(1 + 1)
-        self.failUnlessEqual(combo.get_active_type(), CoverSourcePlugin)
+        assert combo.get_active_type() == object
+        # Check we have a few types (including separator)
+        combo.set_active(5)
+        assert combo.get_active_type()
         combo.destroy()
 
     def test_plugin_prefs(self):
