@@ -6,15 +6,15 @@
 import os
 import shutil
 
-from tests import TestCase, mkdtemp
-
-from gi.repository import Gtk
-
 from quodlibet import config
 from quodlibet.formats import AudioFile
+from quodlibet.library import SongLibrary
+from quodlibet.library.file import FileLibrary
 from quodlibet.player.nullbe import NullPlayer
 from quodlibet.qltk.tracker import SongTracker, FSInterface
-from quodlibet.library import SongLibrary
+from tests import TestCase, mkdtemp, run_gtk_loop
+
+A_PATH = "/dev/null"
 
 
 class TSongTracker(TestCase):
@@ -31,10 +31,6 @@ class TSongTracker(TestCase):
         self.cm = SongTracker(self.w, self.p, self)
         self.current = None
 
-    def do(self):
-        while Gtk.events_pending():
-            Gtk.main_iteration()
-
     def test_destroy(self):
         self.cm.destroy()
 
@@ -44,9 +40,9 @@ class TSongTracker(TestCase):
         self.p.song = self.s1
         self.p.paused = False
         time.sleep(2)
-        self.do()
+        run_gtk_loop()
         self.p.emit('song-ended', self.s1, False)
-        self.do()
+        run_gtk_loop()
         t = time.time()
         self.assertEquals(self.s1["~#playcount"], 1)
         self.assertEquals(self.s1["~#skipcount"], 0)
@@ -54,7 +50,7 @@ class TSongTracker(TestCase):
 
     def test_skip(self):
         self.p.emit('song-ended', self.s1, True)
-        self.do()
+        run_gtk_loop()
         self.assertEquals(self.s1["~#playcount"], 0)
         self.assertEquals(self.s1["~#skipcount"], 1)
         self.failUnless(self.s1["~#lastplayed"], 10)
@@ -62,7 +58,7 @@ class TSongTracker(TestCase):
     def test_error(self):
         self.current = self.p.song = self.s1
         self.p._error('Test error')
-        self.do()
+        run_gtk_loop()
         self.assertEquals(self.s1["~#playcount"], 0)
         self.assertEquals(self.s1["~#skipcount"], 0)
         self.failUnless(self.s1["~#lastplayed"], 10)
@@ -70,7 +66,7 @@ class TSongTracker(TestCase):
     def test_restart(self):
         self.current = self.s1
         self.p.emit('song-ended', self.s1, True)
-        self.do()
+        run_gtk_loop()
         self.assertEquals(self.s1["~#playcount"], 0)
         self.assertEquals(self.s1["~#skipcount"], 0)
 
@@ -83,39 +79,48 @@ class TFSInterface(TestCase):
     def setUp(self):
         self.p = NullPlayer()
         self.dir = mkdtemp()
+        self.lib = FileLibrary()
+        self.song = AudioFile({"~filename": A_PATH, "title": "bar", "~#length": 10})
+        self.lib.add([self.song])
         self.filename = os.path.join(self.dir, "foo")
-        self.fs = FSInterface(self.filename, self.p)
+        self.fs = FSInterface(self.filename, self.p, self.lib)
 
     def tearDown(self):
         self.p.destroy()
+        self.lib.destroy()
         shutil.rmtree(self.dir)
 
-    def do(self):
-        while Gtk.events_pending():
-            Gtk.main_iteration()
-
     def test_init(self):
-        self.do()
+        run_gtk_loop()
         self.failIf(os.path.exists(self.filename))
 
     def test_start(self):
-        self.p.emit('song_started', AudioFile({"woo": "bar", "~#length": 10}))
-        self.do()
+        self.p.emit('song_started', self.song)
+        run_gtk_loop()
         with open(self.filename, "rb") as h:
-            self.failUnless(b"woo=bar\n" in h.read())
+            self.failUnless(b"title=bar\n" in h.read())
 
     def test_song_ended(self):
-        self.p.emit('song-started', AudioFile({"woo": "bar", "~#length": 10}))
-        self.do()
+        self.p.emit('song-started', self.song)
+        run_gtk_loop()
         self.p.emit('song-ended', {}, False)
-        self.do()
+        run_gtk_loop()
         self.failIf(os.path.exists(self.filename))
 
     def test_elapsed(self):
         self.p.seek(123456)
         self.p.emit('song-started', AudioFile({"~#length": 10}))
-        self.do()
+        run_gtk_loop()
         with open(self.filename, "rb") as h:
             contents = h.read()
         assert b"~#elapsed=123.456" in contents
         assert b"~elapsed=2:03\n" in contents
+
+    def test_current_song_changed(self):
+        self.p.song = self.song
+        self.song["title"] = "new!"
+        self.lib.changed([self.song])
+        run_gtk_loop()
+        with open(self.filename, "rb") as h:
+            contents = h.read()
+        assert b"title=new!\n" in contents
